use std::sync::LazyLock;

use eframe::egui::{
    self,
    style::{Selection, Visuals, WidgetVisuals, Widgets},
    Color32, Stroke,
};

pub fn set_visuals(ctx: &egui::Context) {
    let current = ctx.style().visuals.clone();
    let visuals = Visuals {
        override_text_color: Some(STANDARD_THEME.text),
        faint_bg_color: STANDARD_THEME.surface0,
        extreme_bg_color: STANDARD_THEME.surface0,
        window_fill: STANDARD_THEME.base,
        panel_fill: STANDARD_THEME.base,
        window_stroke: Stroke {
            color: STANDARD_THEME.overlay1,
            ..current.window_stroke
        },
        widgets: Widgets {
            noninteractive: make_widget_visual(
                current.widgets.noninteractive,
                STANDARD_THEME.base,
                &STANDARD_THEME,
            ),
            inactive: make_widget_visual(
                current.widgets.inactive,
                STANDARD_THEME.surface0,
                &STANDARD_THEME,
            ),
            hovered: make_widget_visual(
                current.widgets.hovered,
                STANDARD_THEME.light_purple,
                &STANDARD_THEME,
            ),
            active: make_widget_visual(
                current.widgets.active,
                STANDARD_THEME.surface1,
                &STANDARD_THEME,
            ),
            open: make_widget_visual(
                current.widgets.open,
                STANDARD_THEME.surface0,
                &STANDARD_THEME,
            ),
        },
        selection: Selection {
            bg_fill: STANDARD_THEME.purple,
            stroke: Stroke {
                color: STANDARD_THEME.overlay1,
                ..current.selection.stroke
            },
        },
        ..current
    };
    ctx.set_visuals(visuals);
}
fn make_widget_visual(current: WidgetVisuals, bg: Color32, theme: &Theme) -> WidgetVisuals {
    WidgetVisuals {
        bg_fill: bg,
        weak_bg_fill: bg,
        bg_stroke: Stroke {
            color: theme.overlay1,
            ..current.bg_stroke
        },
        fg_stroke: Stroke {
            color: theme.text,
            ..current.fg_stroke
        },
        ..current
    }
}

pub static STANDARD_THEME: LazyLock<Theme> = LazyLock::new(|| {
    Theme::standard()
});
#[derive(Debug)]
pub struct Theme {
    pub base: Color32,
    pub surface0: Color32,
    pub surface1: Color32,
    pub surface2: Color32,
    pub overlay0: Color32,
    pub overlay1: Color32,
    pub overlay2: Color32,
    pub text: Color32,
    pub purple: Color32,
    pub light_purple: Color32,
}
impl Theme {
    fn standard() -> Self {
        const BASE_HUE: f32 = 45.0;
        const BASE_SATURATION: f32 = 40.0;
        Self {
            base: hsl_to_color32(BASE_HUE, BASE_SATURATION, 88.0),
            surface0: hsl_to_color32(BASE_HUE, BASE_SATURATION, 80.0),
            surface1: hsl_to_color32(BASE_HUE, BASE_SATURATION, 75.0),
            surface2: hsl_to_color32(BASE_HUE, BASE_SATURATION, 65.0),
            overlay0: hsl_to_color32(BASE_HUE, BASE_SATURATION, 60.0),
            overlay1: hsl_to_color32(BASE_HUE, BASE_SATURATION, 55.0),
            overlay2: hsl_to_color32(BASE_HUE, BASE_SATURATION, 50.0),
            text: hsl_to_color32(BASE_HUE, BASE_SATURATION, 25.0),
            purple: hsl_to_color32(270.0, 80.0, 80.0),
            light_purple: hsl_to_color32(270.0, 70.0, 90.0),
        }
    }
}

fn hsl_to_rgb(h: f32, s: f32, l: f32) -> (u8, u8, u8) {
    let s = s / 100.0;
    let l = l / 100.0;
    let chroma = (1.0 - f32::abs(2.0 * l - 1.0)) * s;
    let h_prime = h / 60.0;
    let x = chroma * (1.0 - f32::abs((h_prime % 2.0) - 1.0));

    let (r1, g1, b1) = match h_prime {
        _ if h_prime < 1.0 => (chroma, x, 0.0),
        _ if h_prime >= 1.0 && h_prime < 2.0 => (x, chroma, 0.0),
        _ if h_prime >= 2.0 && h_prime < 3.0 => (0.0, chroma, x),
        _ if h_prime >= 3.0 && h_prime < 4.0 => (0.0, x, chroma),
        _ if h_prime >= 4.0 && h_prime < 5.0 => (x, 0.0, chroma),
        _ if h_prime >= 5.0 && h_prime < 6.0 => (chroma, 0.0, x),
        _ => unreachable!(),
    };

    let m = l - (chroma/2.0);
    let (r,g,b) = ((r1 + m) * 255.0, (g1 + m) * 255.0, (b1 + m) * 255.0);
    (r as u8, g as u8, b as u8)
}

fn hsl_to_color32(h: f32, s: f32, l: f32) -> Color32 {
    let (r, g, b) = hsl_to_rgb(h, s, l);
    Color32::from_rgb(r, g, b)
}
