use eframe::egui::{self, text_selection::visuals, NumExt};

use crate::{
    main_state::MainState,
    modals::ModalState,
    transfer_menu::{CurrentTransferState, CurrentTransferStateInterior},
};

use std::sync::LazyLock;

fn tree_label(
    ui: &mut egui::Ui,
    ms: &MainState,
    modal_state: &mut ModalState,
    name: &str,
    uuid: plate_tool_lib::uuid::Uuid,
    selected: bool,
) -> (Option<plate_tool_lib::uuid::Uuid>, bool) {
    let button_padding = ui.spacing().button_padding;
    let total_extra = button_padding + button_padding;
    let wrap_width = ui.available_width() - total_extra.x;
    let galley =
        egui::WidgetText::from(name).into_galley(ui, None, wrap_width, egui::TextStyle::Button);
    let desired_size = {
        let mut desired_size = total_extra + galley.size();
        desired_size.y = desired_size.y.at_least(ui.spacing().interact_size.y);
        desired_size
    };
    let (rect, response) = ui.allocate_at_least(desired_size, egui::Sense::click());
    let text_pos = ui
        .layout()
        .align_size_within_rect(galley.size(), rect.shrink2(button_padding))
        .min;
    let visuals = ui.style().interact_selectable(&response, selected);
    if selected || response.hovered() {
        let rect = rect.expand(visuals.expansion);
        ui.painter().rect(
            rect,
            visuals.rounding,
            visuals.weak_bg_fill,
            visuals.bg_stroke,
        );
    }
    ui.painter().galley(text_pos, galley, visuals.text_color());

    if response.clicked() {
        return (Some(uuid), false);
    }
    if response.double_clicked() {
        return (None, true);
    }
    (None, false)
}

pub fn tree(
    ui: &mut egui::Ui,
    ms: &mut MainState,
    cts: &CurrentTransferState,
    modal_state: &mut ModalState,
) {
    // Add all source plates
    ui.vertical(|ui| {
        ui.heading("Source Plates");
        let mut new_uuid: Option<plate_tool_lib::uuid::Uuid> = None;
        for (name, uuid) in ms.source_plates.iter().map(|x| (&x.name, x.get_uuid())) {
            let (potential_new_uuid, dbl_clicked) = tree_label(
                ui,
                ms,
                modal_state,
                name,
                uuid,
                ms.get_current_source_uuid().is_some_and(|x| x == uuid),
            );
            if potential_new_uuid.is_some() {
                new_uuid = potential_new_uuid;
            }
            if dbl_clicked {
                let pi = ms.get_source_by_uuid(uuid);
                if let Some(pi) = pi {
                    crate::modals::open_edit_plate_modal_plateinstance(modal_state, &pi);
                }
            }
        }
        if let Some(uuid) = new_uuid {
            let current_source_uuid = ms.get_current_source_uuid();
            if current_source_uuid.is_some_and(|x| x != uuid) || current_source_uuid.is_none() {
                ms.set_current_source(uuid);
                ms.set_no_current_transfer();
                if let Some(mut cts) = cts.lock().ok() {
                    *cts = CurrentTransferStateInterior::default();
                }
                set_plates(ms, cts);
            }
        }
    });

    // Add all destination plates
    ui.vertical(|ui| {
        ui.heading("Destination Plates");
        let mut new_uuid: Option<plate_tool_lib::uuid::Uuid> = None;
        for (name, uuid) in ms
            .destination_plates
            .iter()
            .map(|x| (&x.name, x.get_uuid()))
        {
            let (potential_new_uuid, dbl_clicked) = tree_label(
                ui,
                ms,
                modal_state,
                name,
                uuid,
                ms.get_current_destination_uuid().is_some_and(|x| x == uuid),
            );
            if potential_new_uuid.is_some() {
                new_uuid = potential_new_uuid;
            }
            if dbl_clicked {
                let pi = ms.get_destination_by_uuid(uuid);
                if let Some(pi) = pi {
                    crate::modals::open_edit_plate_modal_plateinstance(modal_state, &pi);
                }
            }
        }
        if let Some(uuid) = new_uuid {
            let current_destination_uuid = ms.get_current_destination_uuid();
            if current_destination_uuid.is_some_and(|x| x != uuid)
                || current_destination_uuid.is_none()
            {
                ms.set_current_destination(uuid);
                ms.set_no_current_transfer();
                if let Some(mut cts) = cts.lock().ok() {
                    *cts = CurrentTransferStateInterior::default();
                    cts.destination_plate = ms
                        .get_current_destination_plateinstance()
                        .expect("Just set destination")
                        .plate;
                }
                set_plates(ms, cts);
                log::warn!(
                    "{:?}\n{:?}",
                    cts,
                    ms.get_current_destination_plateinstance()
                );
            }
        }
    });
    // Add all transfers
    ui.vertical(|ui| {
        ui.heading("Transfers");
        let mut new_uuid: Option<plate_tool_lib::uuid::Uuid> = None;
        for (name, uuid) in ms.transfers.iter().map(|x| (&x.name, x.get_uuid())) {
            let (potential_new_uuid, _) = tree_label(
                ui,
                ms,
                modal_state,
                name,
                uuid,
                ms.get_current_transfer_uuid().is_some_and(|x| x == uuid),
            );
            if potential_new_uuid.is_some() {
                new_uuid = potential_new_uuid;
            }
        }
        if let Some(uuid) = new_uuid {
            ms.set_current_transfer(uuid);
            if let Some(new_cts) =
                CurrentTransferStateInterior::try_new_from_main_state_transfer(ms)
            {
                *cts.lock().unwrap() = new_cts;
            }
            set_plates(ms, cts);
        }
    });
}

fn set_plates(ms: &MainState, cts: &CurrentTransferState) {
    if let Some(mut cts) = cts.lock().ok() {
        if let Some(source_plate) = ms.get_current_source_plateinstance() {
            cts.source_plate = source_plate.plate;
        }
        if let Some(destination_plate) = ms.get_current_destination_plateinstance() {
            cts.destination_plate = destination_plate.plate;
        }
    }
}
